/*
 * Copyright (c) 2006, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.registry.jdbc.mediatypes.builtin;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.registry.Artifact;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;
import org.wso2.registry.User;
import org.wso2.registry.i18n.Messages;
import org.wso2.registry.jdbc.ConnectionFactory;
import org.wso2.registry.jdbc.dao.CommentsDAO;
import org.wso2.registry.jdbc.dao.RatingsDAO;
import org.wso2.registry.jdbc.dao.TagsDAO;
import org.wso2.registry.jdbc.mediatypes.MediaTypeHandler;
import org.wso2.registry.jdbc.mediatypes.MediaTypeManager;
import org.wso2.usermanager.Realm;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Iterator;
import java.util.List;

public class DefaultMediaTypeHandler extends MediaTypeHandler {

    private static final Log log = LogFactory.getLog(DefaultMediaTypeHandler.class);

    private TagsDAO tagsDAO = new TagsDAO();
    private CommentsDAO commentsDAO = new CommentsDAO();
    private RatingsDAO ratingsDAO = new RatingsDAO();

    public DefaultMediaTypeHandler(ConnectionFactory connectionFactory,
                                   Realm realm,
                                   MediaTypeManager mediaTypeManager) {

        super(connectionFactory, realm, mediaTypeManager);
    }

    public Artifact get(String path, Artifact artifact) throws RegistryException {

        String plainPath = path;
        long versionNumber = -1;
        if (path.indexOf("?v=") != -1) {
            String[] parts = path.split("\\?v=");
            plainPath = parts[0];
            versionNumber = new Long(parts[1]).longValue();
        }

        Connection conn = connectionFactory.getConnection();
        try {
            //artifact = artifactDAO.get(path, conn);

            if (versionNumber == -1) {
                artifact = artifactDAO.getLatestVersion(plainPath, conn);
            } else {
                artifact = artifactDAO.get(plainPath, versionNumber, conn);
            }

        } catch (SQLException e) {

            String msg = Messages.getMessage("resource.get.fail", path);
            log.error(msg, e);
            throw new RegistryException(msg);

        } finally {
            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }

        return artifact;
    }

    public boolean put(String path, Artifact artifact) throws RegistryException {

        String userID = User.getCurrentUser();

        Connection conn = connectionFactory.getConnection();

        try {
            conn.setAutoCommit(false);

            Artifact a = artifactDAO.getLatestVersion(path, conn);
            if (a != null) {

                // given artifact already exist. update it.
                artifact.setId(a.getId());

                // if client has not explicitly set the author, keep the original author name.
                if (artifact.getAuthorUserName() == null) {
                    artifact.setAuthorUserName(a.getAuthorUserName());
                }

                artifact.setLastUpdaterUserName(userID);
                artifactDAO.update(path, artifact, conn);
                artifactDAO.addResourceVersion(artifact, conn);
            } else {

                // given artifact does not exist. add it as a new artifact.

                // first create all ascendant collections
                Artifact currentParent = artifactDAO.
                        getLatestVersion(RegistryConstants.ROOT_PATH, conn);
                String[] parts = path.split(RegistryConstants.PATH_SEPARATOR);

                String currentPath = RegistryConstants.ROOT_PATH + parts[1];
                for (int i = 2; i < parts.length; i++) {

                    Artifact currentCollection = artifactDAO.getLatestVersion(currentPath, conn);
                    if (currentCollection == null) {

                        currentCollection = new Artifact();
                        currentCollection.setPath(currentPath);
                        currentCollection.setDirectory(true);
                        currentCollection.setAuthorUserName(userID);
                        currentCollection.setLastUpdaterUserName(userID);

                        artifactDAO.add(currentPath, currentCollection, conn);
                        authorizationUtil.setDefaultAuthorizations(realm, currentPath, userID);

                        currentCollection = artifactDAO.getLatestVersion(currentPath, conn);

                        // version the new collection
                        //VersionHandler.addResourceToVersionTable(conn, currentCollection);

                        // add new collection as a child of the current parent
                        //artifactDAO.addChild(
                        //        currentParent.getId(), currentCollection.getId(), conn);

                    } else {

                        if (!currentCollection.isDirectory()) {
                            String message =
                                    Messages.getMessage("artifact.adding.error", currentPath, path);
                            throw new RegistryException(message);
                        }
                    }

                    currentParent = currentCollection;
                    currentPath = currentPath + RegistryConstants.PATH_SEPARATOR + parts[i];
                }

                // now add the required artifact
                artifact.setPath(path);
                if (artifact.getAuthorUserName() == null) {
                    artifact.setAuthorUserName(userID);
                }
                artifact.setLastUpdaterUserName(userID);

                artifactDAO.add(path, artifact, conn);
                authorizationUtil.setDefaultAuthorizations(realm, path, userID);

                artifact.setId(artifactDAO.getArtifactID(path, conn));
                artifactDAO.addResourceVersion(artifact, conn);

                // add new artifact as a child of the current parent
                //artifactDAO.addChild(currentParent.getId(), artifact.getId(), conn);
            }

            conn.commit();

        } catch (SQLException e) {

            try {
                conn.rollback();
            } catch (SQLException e1) {
                e1.printStackTrace();
            }

            String msg = Messages.getMessage("resource.put.fail", path);
            log.error(msg, e);
            throw new RegistryException(msg);

        } finally {

            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }

        return true;
    }

    public boolean delete(String path) throws RegistryException {

        Connection conn = connectionFactory.getConnection();

        try {
            conn.setAutoCommit(false);

            Artifact artifact = artifactDAO.getLatestVersion(path, conn);
            if (artifact != null) {
                if (artifact.isDirectory()) {
                    deleteDirectory(artifact, conn);

                } else {
                    artifactDAO.markDeleted(path, conn);
                    artifactDAO.deleteResource(artifact, conn);

                    //prepareToDelete(artifact.getId(), conn);
                    //artifactDAO.delete(artifact.getPath(), conn);
                }
            } else {
                String msg = Messages.getMessage("delete.null.resource", path);
                log.error(msg);
                throw new RegistryException(msg);
            }

            conn.commit();

        } catch (SQLException e) {

            try {
                conn.rollback();
            } catch (SQLException e1) {
                e1.printStackTrace();
            }

            String msg = Messages.getMessage("resource.delete.fail", path);
            log.error(msg, e);
            throw new RegistryException(msg);

        } finally {

            try {
                conn.close();
            } catch (SQLException e) {
                e.printStackTrace();
            }
        }

        return true;
    }

    private void deleteDirectory(Artifact collection, Connection conn) throws SQLException {

        List childList = artifactDAO.getChildren(collection.getId(), -1, conn);
        Iterator i = childList.iterator();
        while (i.hasNext()) {
            Artifact child = (Artifact) i.next();

            if (child.isDirectory()) {
                deleteDirectory(child, conn);
            } else {
                //prepareToDelete(child.getId(), conn);
                //artifactDAO.delete(child.getPath(), conn);

                artifactDAO.markDeleted(child.getPath(), conn);
                artifactDAO.deleteResource(child, conn);
            }
        }

        //prepareToDelete(collection.getId(), conn);
        //artifactDAO.deleteByID(collection.getId(), conn);
        
        artifactDAO.markDeleted(collection.getPath(), conn);
        artifactDAO.deleteResource(collection, conn);
    }

    //private void prepareToDelete(long artfactID, Connection conn) throws SQLException {
    //
    //    // remove properties
    //    artifactDAO.removeProperties(artfactID, conn);
    //
    //    // remove child relationships
    //    artifactDAO.removeChildRelationship(artfactID, conn);
    //
    //    // remove taggings
    //    tagsDAO.removeTaggings(artfactID, conn);
    //
    //    // remove comments
    //    commentsDAO.removeComments(artfactID, conn);
    //
    //    // remove ratings
    //    ratingsDAO.removeRatings(artfactID, conn);
    //}
}
